from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Friend
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import FriendMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    ArgResult,
    RegexMatch,
    ArgumentMatch,
)

from ....core.control import Permission
from ....core.subgroup_config import SubGroup

channel = Channel.current()

ADD_ERROR_MSG = """增加失败, 请正确输入!
请按照 [指令 --name 组名 --uids UID] 格式进行添加

指令：
(添加|增加)(主播|[uU][pP])到订阅组

参数：
--name 订阅组名称
--uids up主的uid, 多个uid以空格分割

例:
增加主播到订阅组 --name XXX --uids 123456
添加up到订阅组 --name XXX --uids 123456 234567
"""


@channel.use(
    ListenerSchema(
        listening_events=[FriendMessage],
        inline_dispatchers=[
            Twilight(
                [
                    RegexMatch(r"(添加|增加)(主播|[uU][pP])到订阅组"),
                    "group_name" @ ArgumentMatch("--name", optional=True, nargs="*", type=str),
                    "sublist" @ ArgumentMatch("--uids", optional=True, nargs="*", type=int),
                ],
            )
        ],
    )
)
async def main(app: Ariadne, friend: Friend, group_name: ArgResult, sublist: ArgResult):
    """
    通过指定的命令进行添加up主到对应的订阅组中

    Attributes:
        group_name: 要添加的订阅组名称
        sublist: 要添加的订阅列表
    """
    Permission.manual(friend, Permission.MASTER)

    if group_name.matched and sublist.matched:
        group_name = group_name.result[0]
        uid_list: list = sublist.result
        sg = SubGroup(group_name)
        if sg.is_in_group_names():
            if sg.add_to_subGroups(uid_list):
                await app.send_friend_message(friend, MessageChain(f"添加到订阅组 [{group_name}] 成功"))
            else:
                await app.send_friend_message(
                    friend,
                    MessageChain(f"添加失败, 订阅组 [{group_name}] 添加后up主数量将超过12个!"),
                )
        else:
            await app.send_friend_message(friend, MessageChain("添加的订阅组名称不存在!"))
    else:
        await app.send_friend_message(friend, MessageChain(ADD_ERROR_MSG))



from typing import Optional
from fastapi import FastAPI
from graia.amnesia.builtins.starlette import StarletteService


class FastAPIService(StarletteService):
    def __init__(self, fastapi: Optional[FastAPI] = None) -> None:
        self.fastapi = fastapi or FastAPI()
        super().__init__(self.fastapi)



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group
from graia.ariadne.message.element import At
from graia.ariadne.event.message import GroupMessage
from graia.ariadne.message.chain import MessageChain
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    RegexMatch,
    ParamMatch,
    RegexResult,
    ElementMatch,
    ElementResult,
)

from ....core.bot_config import BotConfig
from ....utils.up_operation import set_nick
from ....utils.uid_extract import uid_extract
from ....core.control import Interval, Permission

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[
            Twilight(
                [
                    "at" @ ElementMatch(At, optional=True),
                    "act" @ RegexMatch(r"添加|修改|设定|删除"),
                    RegexMatch(r"昵称|别名"),
                    "uid" @ ParamMatch(optional=True),
                    "nick" @ ParamMatch(optional=True),
                ]
            )
        ],
        decorators=[Permission.require(Permission.GROUP_ADMIN), Interval.require()],
    )
)
async def main(
    app: Ariadne,
    group: Group,
    at: ElementResult,
    act: RegexResult,
    uid: RegexResult,
    nick: RegexResult,
):
    if at.result:
        at_element: At = at.result  # type: ignore
        if at_element.target != BotConfig.Mirai.account:
            return

    if uid.matched:
        uid = await uid_extract(uid.result.display, group.id)
        if uid:
            acts = act.result.display
            if acts in ["设定", "修改", "添加"]:
                nicks = nick.result.display
                if len(nicks) > 24:
                    msg = "昵称过长，设定失败"
                else:
                    msg = f"{uid} 昵称设定成功" if set_nick(uid, group.id, nicks) else "该群未关注此 UP"
            else:
                msg = f"{uid} 昵称删除成功" if set_nick(uid, group.id, None) else "该群未关注此 UP"
        else:
            msg = "请输入正确的 UID"
    else:
        msg = "请输入正确的 UID"

    await app.send_group_message(group, MessageChain(msg))



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Friend
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import FriendMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import Twilight, FullMatch

from ....core.control import Permission
from ....core.bot_config import BotConfig

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[FriendMessage],
        inline_dispatchers=[
            Twilight(
                [FullMatch("关闭白名单")],
            )
        ],
    )
)
async def main(app: Ariadne, friend: Friend):
    Permission.manual(friend, Permission.MASTER)
    if BotConfig.access_control:
        BotConfig.access_control = False
        BotConfig.save()
        msg = "白名单关闭成功"
    else:
        msg = "白名单当前已关闭"
    await app.send_friend_message(friend, MessageChain(msg))



import sys
import contextlib


def detect_package():
    try:
        if __nuitka_binary_dir is not None:  # type: ignore
            return "nuitka"
    except NameError:
        return (
            "pyinstaller"
            if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS")
            else False
        )
    return False


is_package = detect_package()

if is_package == "nuitka":
    from creart import add_creator
    from importlib_metadata import EntryPoint

    _entrys = [
        {
            "name": "broadcast",
            "value": "graia.creart.broadcast:BroadcastCreator",
            "group": "creart.creators",
        },
        {
            "name": "broadcast_behaviour",
            "value": "graia.creart.broadcast:BroadcastBehaviourCreator",
            "group": "creart.creators",
        },
        {
            "name": "loop",
            "value": "graia.creart.broadcast:EventLoopCreator",
            "group": "creart.creators",
        },
        {"name": "saya", "value": "graia.creart.saya:SayaCreator", "group": "creart.creators"},
        {
            "name": "scheduler",
            "value": "graia.creart.scheduler:SchedulerCreator",
            "group": "creart.creators",
        },
        {
            "name": "scheduler_behaviour",
            "value": "graia.creart.scheduler:SchedulerBehaviourCreator",
            "group": "creart.creators",
        },
        {
            "name": "commander",
            "value": "graia.ariadne.message.commander.creart:CommanderCreator",
            "group": "creart.creators",
        },
        {
            "name": "commander_behaviour",
            "value": "graia.ariadne.message.commander.creart:CommanderBehaviourCreator",
            "group": "creart.creators",
        },
    ]
    for entry in _entrys:
        with contextlib.suppress(Exception):
            creator = EntryPoint(**entry).load()
            if creator.available():
                add_creator(creator)


def detect_playwright():
    try:
        import playwright  # type: ignore # noqa

        return True
    except ImportError:
        return False


is_full = detect_playwright()



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Friend
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import FriendMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import Twilight, RegexMatch

from ....core.subgroup_config import get_subgroup_list
from ....core.control import Permission

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[FriendMessage],
        inline_dispatchers=[Twilight(RegexMatch(r"(查看)?订阅组(列表)?"))],
    )
)
async def sub_list(app: Ariadne, friend: Friend):
    Permission.manual(friend, Permission.MASTER)

    group_names, sub_lists = get_subgroup_list()
    group_name_count = len(group_names)
    if group_name_count == 0:
        await app.send_friend_message(friend, MessageChain("当前未创建订阅组"))
    else:
        # TODO 需要考虑是否增加up主名称显示
        msg = [f"当前共创建订阅组 {group_name_count} 个"]
        for i, data in enumerate(group_names, 1):
            group_name = data
            sublist = sub_lists[i - 1]
            msg.append(f"\n{i}. {f'{group_name}'}（{sublist}）")
        await app.send_friend_message(friend, MessageChain(msg))



import contextlib

from loguru import logger
from fastapi import WebSocket, APIRouter
from starlette.websockets import WebSocketDisconnect
from websockets.exceptions import ConnectionClosedOK

from ....core.control import Permission
from .auth import verify_token, UnauthorizedException


router = APIRouter(tags=["Logs"])


@router.websocket("/logs", name="logs")
async def websocket_endpoint(websocket: WebSocket):
    await websocket.accept()

    async def log_handler(message: str):
        try:
            await websocket.send_json({"type": "log", "message": message})
        except Exception:
            with contextlib.suppress():
                await websocket.close()

    log = None
    log_level = None

    try:
        while True:
            data = await websocket.receive_json()
            if data["token"] is None:
                raise UnauthorizedException
            try:
                oauth2 = await verify_token(data["token"])
                if oauth2.permission < Permission.MASTER:
                    raise UnauthorizedException(detail="Permission denied")
                logger.info(f"[WS] Auth success, {oauth2.qq}")
            except UnauthorizedException as e:
                await websocket.send_json(
                    {
                        "type": "sys",
                        "status": "error",
                        "message": e.detail,
                    }
                )
                logger.warning("[WS] Auth failed, Permission denied")
                return await websocket.close()
            if data.get("status", "") == "start":
                if (level := data.get("level", "").upper()) in [
                    "DEBUG",
                    "INFO",
                    "WARNING",
                    "ERROR",
                    "CRITICAL",
                ]:
                    if log_level != level:
                        if log:
                            logger.remove(log)
                        log = logger.add(log_handler, level=level, colorize=False)
                        await websocket.send_json(
                            {"type": "sys", "status": "success", "level": level}
                        )
                        logger.info(f"[WS] Log transfer started, level: {level}")
                    else:
                        await websocket.send_json(
                            {
                                "type": "sys",
                                "status": "ignore",
                                "message": f"Log level is already {level}",
                            }
                        )
                else:
                    logger.warning(f"[WS] Invalid log level {level}")
                    await websocket.send_json(
                        {
                            "type": "sys",
                            "status": "error",
                            "message": "Invalid log level",
                        }
                    )
            elif data.get("status", "") == "stop":
                if log:
                    logger.remove(log)
                    log = None
                    await websocket.send_json(
                        {"type": "sys", "status": "success", "message": "Log transfer stopped"}
                    )
                    logger.info("[WS] Log transfer stopped")
                else:
                    await websocket.send_json(
                        {
                            "type": "sys",
                            "status": "error",
                            "message": "Log transfer not started",
                        }
                    )

    except (WebSocketDisconnect, ConnectionClosedOK):
        pass
    except Exception as e:
        with contextlib.suppress():
            await websocket.send_json({"type": "sys", "status": "error", "message": str(e)})
        with contextlib.suppress():
            await websocket.close()
        logger.error(e)
    if log:
        logger.remove(log)
